@@ -64,9 +64,18 @@ ignored_functions = {
   "virDomainMigrateConfirm3": "private function for migration",
   "virDomainMigratePrepareTunnel3": "private function for tunnelled migration",
   "DllMain": "specific function for Win32",
-  "virTypedParameterArrayValidate": "internal function in virtypedparam.c",
+  "virTypedParamsValidate": "internal function in virtypedparam.c",
   "virTypedParameterAssign": "internal function in virtypedparam.c",
   "virTypedParameterAssignFromStr": "internal function in virtypedparam.c",
+  "virTypedParameterToString": "internal function in virtypedparam.c",
+  "virTypedParamsCheck": "internal function in virtypedparam.c",
+  "virTypedParamsCopy": "internal function in virtypedparam.c",
+  "virDomainMigrateBegin3Params": "private function for migration",
+  "virDomainMigrateFinish3Params": "private function for migration",
+  "virDomainMigratePerform3Params": "private function for migration",
+  "virDomainMigratePrepare3Params": "private function for migration",
+  "virDomainMigrateConfirm3Params": "private function for migration",
+  "virDomainMigratePrepareTunnel3Params": "private function for tunnelled migration",
 }
 
 ignored_macros = {
