@@ -118,6 +118,8 @@ typedef enum {
     VIR_FROM_IDENTITY = 53,     /* Error from identity code */
     VIR_FROM_CGROUP = 54,       /* Error from cgroups */
 
+    VIR_FROM_ACCESS = 55,       /* Error from access control manager */
+
 # ifdef VIR_ENUM_SENTINELS
     VIR_ERR_DOMAIN_LAST
 # endif
@@ -290,6 +292,8 @@ typedef enum {
     VIR_ERR_AGENT_UNRESPONSIVE = 86,    /* guest agent is unresponsive,
                                            not running or not usable */
     VIR_ERR_RESOURCE_BUSY = 87,         /* resource is already in use */
+    VIR_ERR_ACCESS_DENIED = 88,         /* operation on the object/resource
+                                           was denied */
 } virErrorNumber;
 
 /**
