@@ -25,10 +25,10 @@
 
 #include "virerror.h"
 #include "viralloc.h"
-#include "virutil.h"
 #include "virbuffer.h"
 #include "cpu_conf.h"
 #include "domain_conf.h"
+#include "virstring.h"
 
 #define VIR_FROM_THIS VIR_FROM_CPU
 
@@ -81,7 +81,7 @@ virCPUDefFree(virCPUDefPtr def)
 
     virCPUDefFreeModel(def);
 
-    for (i = 0 ; i < def->ncells ; i++) {
+    for (i = 0; i < def->ncells; i++) {
         virBitmapFree(def->cells[i].cpumask);
         VIR_FREE(def->cells[i].cpustr);
     }
@@ -99,10 +99,10 @@ virCPUDefCopyModel(virCPUDefPtr dst,
 {
     unsigned int i;
 
-    if ((src->model && !(dst->model = strdup(src->model)))
-        || (src->vendor && !(dst->vendor = strdup(src->vendor)))
-        || (src->vendor_id && !(dst->vendor_id = strdup(src->vendor_id)))
-        || VIR_ALLOC_N(dst->features, src->nfeatures) < 0)
+    if (VIR_STRDUP(dst->model, src->model) < 0 ||
+        VIR_STRDUP(dst->vendor, src->vendor) < 0 ||
+        VIR_STRDUP(dst->vendor_id, src->vendor_id) < 0 ||
+        VIR_ALLOC_N(dst->features, src->nfeatures) < 0)
         goto no_memory;
     dst->nfeatures_max = dst->nfeatures = src->nfeatures;
 
@@ -118,8 +118,8 @@ virCPUDefCopyModel(virCPUDefPtr dst,
             dst->features[i].policy = src->features[i].policy;
         }
 
-        if (!(dst->features[i].name = strdup(src->features[i].name)))
-            goto no_memory;
+        if (VIR_STRDUP(dst->features[i].name, src->features[i].name) < 0)
+            return -1;
     }
 
     return 0;
@@ -167,8 +167,8 @@ virCPUDefCopy(const virCPUDefPtr cpu)
             if (!copy->cells[i].cpumask)
                 goto no_memory;
 
-            if (!(copy->cells[i].cpustr = strdup(cpu->cells[i].cpustr)))
-                goto no_memory;
+            if (VIR_STRDUP(copy->cells[i].cpustr, cpu->cells[i].cpustr) < 0)
+                goto error;
         }
         copy->cells_cpus = cpu->cells_cpus;
     }
@@ -387,7 +387,7 @@ virCPUDefParseXML(const xmlNodePtr node,
         def->nfeatures = n;
     }
 
-    for (i = 0 ; i < n ; i++) {
+    for (i = 0; i < n; i++) {
         char *name;
         int policy; /* enum virDomainCPUFeaturePolicy */
         unsigned int j;
@@ -418,7 +418,7 @@ virCPUDefParseXML(const xmlNodePtr node,
             goto error;
         }
 
-        for (j = 0 ; j < i ; j++) {
+        for (j = 0; j < i; j++) {
             if (STREQ(name, def->features[j].name)) {
                 virReportError(VIR_ERR_INTERNAL_ERROR,
                                _("CPU feature `%s' specified more than once"),
@@ -447,7 +447,7 @@ virCPUDefParseXML(const xmlNodePtr node,
 
         def->ncells = n;
 
-        for (i = 0 ; i < n ; i++) {
+        for (i = 0; i < n; i++) {
             char *cpus, *memory;
             int ret, ncpus = 0;
 
@@ -630,7 +630,7 @@ virCPUDefFormatBuf(virBufferPtr buf,
     }
 
     if (formatModel) {
-        for (i = 0 ; i < def->nfeatures ; i++) {
+        for (i = 0; i < def->nfeatures; i++) {
             virCPUFeatureDefPtr feature = def->features + i;
 
             if (!feature->name) {
@@ -679,7 +679,7 @@ virCPUDefAddFeature(virCPUDefPtr def,
 {
     int i;
 
-    for (i = 0 ; i < def->nfeatures ; i++) {
+    for (i = 0; i < def->nfeatures; i++) {
         if (STREQ(name, def->features[i].name)) {
             virReportError(VIR_ERR_INTERNAL_ERROR,
                            _("CPU feature `%s' specified more than once"), name);
@@ -694,8 +694,8 @@ virCPUDefAddFeature(virCPUDefPtr def,
     if (def->type == VIR_CPU_TYPE_HOST)
         policy = -1;
 
-    if (!(def->features[def->nfeatures].name = strdup(name)))
-        goto no_memory;
+    if (VIR_STRDUP(def->features[def->nfeatures].name, name) < 0)
+        return -1;
 
     def->features[def->nfeatures].policy = policy;
     def->nfeatures++;
@@ -796,7 +796,7 @@ virCPUDefIsEqual(virCPUDefPtr src,
         goto cleanup;
     }
 
-    for (i = 0 ; i < src->nfeatures ; i++) {
+    for (i = 0; i < src->nfeatures; i++) {
         if (STRNEQ(src->features[i].name, dst->features[i].name)) {
             virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
                            _("Target CPU feature %s does not match source %s"),
