@@ -24,9 +24,9 @@
 
 #include "netdev_bandwidth_conf.h"
 #include "virerror.h"
-#include "virutil.h"
 #include "viralloc.h"
 #include "domain_conf.h"
+#include "virstring.h"
 
 #define VIR_FROM_THIS VIR_FROM_NONE
 
