@@ -17,6 +17,7 @@
  *
  * Authors:
  *     Laine Stump <laine@redhat.com>
+ *     James Robson <jrobson@websense.com>
  */
 
 #include <config.h>
@@ -27,12 +28,16 @@
 
 #define VIR_FROM_THIS VIR_FROM_NONE
 
+VIR_ENUM_IMPL(virNativeVlanMode, VIR_NATIVE_VLAN_MODE_LAST,
+              "default", "tagged", "untagged")
+
 int
 virNetDevVlanParse(xmlNodePtr node, xmlXPathContextPtr ctxt, virNetDevVlanPtr def)
 {
     int ret = -1;
     xmlNodePtr save = ctxt->node;
     const char *trunk = NULL;
+    const char *nativeMode = NULL;
     xmlNodePtr *tagNodes = NULL;
     int nTags, ii;
 
@@ -40,20 +45,22 @@ virNetDevVlanParse(xmlNodePtr node, xmlXPathContextPtr ctxt, virNetDevVlanPtr de
 
     nTags = virXPathNodeSet("./tag", ctxt, &tagNodes);
     if (nTags < 0)
-        goto error;
+        goto cleanup;
 
     if (nTags == 0) {
         virReportError(VIR_ERR_XML_ERROR, "%s",
                        _("missing tag id - each <vlan> must have "
                          "at least one <tag id='n'/> subelement"));
-        goto error;
+        goto cleanup;
     }
 
     if (VIR_ALLOC_N(def->tag, nTags) < 0) {
         virReportOOMError();
-        goto error;
+        goto cleanup;
     }
 
+    def->nativeMode = 0;
+    def->nativeTag = 0;
     for (ii = 0; ii < nTags; ii++) {
         unsigned long id;
 
@@ -61,12 +68,29 @@ virNetDevVlanParse(xmlNodePtr node, xmlXPathContextPtr ctxt, virNetDevVlanPtr de
         if (virXPathULong("string(./@id)", ctxt, &id) < 0) {
             virReportError(VIR_ERR_XML_ERROR, "%s",
                            _("missing or invalid vlan tag id attribute"));
-            goto error;
+            goto cleanup;
         }
         if (id > 4095) {
             virReportError(VIR_ERR_XML_ERROR,
                            _("vlan tag id %lu too large (maximum 4095)"), id);
-            goto error;
+            goto cleanup;
+        }
+        if ((nativeMode = virXPathString("string(./@nativeMode)", ctxt))) {
+            if (def->nativeMode != 0) {
+                virReportError(VIR_ERR_XML_ERROR, "%s",
+                               _("duplicate native vlan setting"));
+                goto cleanup;
+            }
+            if ((def->nativeMode
+                 = virNativeVlanModeTypeFromString(nativeMode)) <= 0) {
+                virReportError(VIR_ERR_XML_ERROR,
+                               _("Invalid \"nativeMode='%s'\" "
+                                 "in vlan <tag> element"),
+                               nativeMode);
+                goto cleanup;
+            }
+            VIR_FREE(nativeMode);
+            def->nativeTag = id;
         }
         def->tag[ii] = id;
     }
@@ -87,23 +111,30 @@ virNetDevVlanParse(xmlNodePtr node, xmlXPathContextPtr ctxt, virNetDevVlanPtr de
                 virReportError(VIR_ERR_XML_ERROR,
                                _("invalid \"trunk='%s'\" in <vlan> - trunk='yes' "
                                  "is required for more than one vlan tag"), trunk);
-                goto error;
+                goto cleanup;
+            }
+            if (def->nativeMode != 0) {
+                virReportError(VIR_ERR_XML_ERROR, "%s",
+                               _("invalid configuration in <vlan> - \"trunk='no'\" is "
+                                 "not allowed with a native vlan id"));
+                goto cleanup;
             }
             /* allow (but discard) "trunk='no' if there is a single tag */
             if (STRCASENEQ(trunk, "no")) {
                 virReportError(VIR_ERR_XML_ERROR,
                                _("invalid \"trunk='%s'\" in <vlan> "
                                  "- must be yes or no"), trunk);
-                goto error;
+                goto cleanup;
             }
         }
     }
 
     ret = 0;
-error:
+cleanup:
     ctxt->node = save;
     VIR_FREE(tagNodes);
     VIR_FREE(trunk);
+    VIR_FREE(nativeMode);
     if (ret < 0)
         virNetDevVlanClear(def);
     return ret;
@@ -125,7 +156,19 @@ virNetDevVlanFormat(virNetDevVlanPtr def, virBufferPtr buf)
 
     virBufferAsprintf(buf, "<vlan%s>\n", def->trunk ? " trunk='yes'" : "");
     for (ii = 0; ii < def->nTags; ii++) {
-        virBufferAsprintf(buf, "  <tag id='%u'/>\n", def->tag[ii]);
+        if (def->nativeMode != VIR_NATIVE_VLAN_MODE_DEFAULT &&
+            def->nativeTag == def->tag[ii]) {
+            /* check the nativeMode in case we get <tag id='0'/>*/
+            const char *mode = virNativeVlanModeTypeToString(def->nativeMode);
+            if (!mode) {
+                virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                               _("Bad value for nativeMode"));
+            }
+            virBufferAsprintf(buf, "  <tag id='%u' nativeMode='%s'/>\n",
+                              def->tag[ii], mode);
+        } else {
+            virBufferAsprintf(buf, "  <tag id='%u'/>\n", def->tag[ii]);
+        }
     }
     virBufferAddLit(buf, "</vlan>\n");
     return 0;
