@@ -629,11 +629,11 @@ static int lxcContainerPivotRoot(virDomainFSDefPtr root)
         }
     }
 
-    /* Now we chroot into the tmpfs, then pivot into the
+    /* Now we chdir into the tmpfs, then pivot into the
      * root->src bind-mounted onto '/new' */
     if (chdir(newroot) < 0) {
         virReportSystemError(errno,
-                             _("Failed to chroot into %s"), newroot);
+                             _("Failed to chdir into %s"), newroot);
         goto err;
     }
 
@@ -869,25 +869,14 @@ static int lxcContainerPopulateDevices(char **ttyPaths, size_t nttyPaths)
         }
     }
 
-    if (access("/dev/pts/ptmx", W_OK) == 0) {
-        /* We have private devpts capability, so bind that */
-        if (virFileTouch("/dev/ptmx", 0666) < 0)
-            return -1;
+    /* We have private devpts capability, so bind that */
+    if (virFileTouch("/dev/ptmx", 0666) < 0)
+        return -1;
 
-        if (mount("/dev/pts/ptmx", "/dev/ptmx", "ptmx", MS_BIND, NULL) < 0) {
-            virReportSystemError(errno, "%s",
-                                 _("Failed to bind /dev/pts/ptmx on to /dev/ptmx"));
-            return -1;
-        }
-    } else {
-        /* Legacy devpts, so we need to just use shared one */
-        dev_t dev = makedev(LXC_DEV_MAJ_TTY, LXC_DEV_MIN_PTMX);
-        if (mknod("/dev/ptmx", S_IFCHR, dev) < 0 ||
-            chmod("/dev/ptmx", 0666)) {
-            virReportSystemError(errno, "%s",
-                                 _("Failed to make device /dev/ptmx"));
-            return -1;
-        }
+    if (mount("/dev/pts/ptmx", "/dev/ptmx", "ptmx", MS_BIND, NULL) < 0) {
+        virReportSystemError(errno, "%s",
+                             _("Failed to bind /dev/pts/ptmx on to /dev/ptmx"));
+        return -1;
     }
 
     for (i = 0; i < nttyPaths; i++) {
