@@ -1,7 +1,7 @@
 /*
  * virkeyfile.c: "ini"-style configuration file handling
  *
- * Copyright (C) 2012 Red Hat, Inc.
+ * Copyright (C) 2012-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,10 +28,12 @@
 #include "c-ctype.h"
 #include "virlog.h"
 #include "viralloc.h"
+#include "virfile.h"
 #include "virutil.h"
 #include "virhash.h"
 #include "virkeyfile.h"
 #include "virerror.h"
+#include "virstring.h"
 
 #define VIR_FROM_THIS VIR_FROM_CONF
 
@@ -121,10 +123,8 @@ static int virKeyFileParseGroup(virKeyFileParserCtxtPtr ctxt)
         return -1;
     }
 
-    if (!(ctxt->groupname = strndup(name, ctxt->cur - name))) {
-        virReportOOMError();
+    if (VIR_STRNDUP(ctxt->groupname, name, ctxt->cur - name) < 0)
         return -1;
-    }
 
     NEXT;
 
@@ -167,10 +167,8 @@ static int virKeyFileParseValue(virKeyFileParserCtxtPtr ctxt)
         return -1;
     }
 
-    if (!(key = strndup(keystart, ctxt->cur - keystart))) {
-        virReportOOMError();
+    if (VIR_STRNDUP(key, keystart, ctxt->cur - keystart) < 0)
         return -1;
-    }
 
     NEXT;
     valuestart = ctxt->cur;
@@ -183,10 +181,8 @@ static int virKeyFileParseValue(virKeyFileParserCtxtPtr ctxt)
     len = ctxt->cur - valuestart;
     if (IS_EOF && !IS_EOL(CUR))
         len++;
-    if (!(value = strndup(valuestart, len))) {
-        virReportOOMError();
+    if (VIR_STRNDUP(value, valuestart, len) < 0)
         goto cleanup;
-    }
 
     if (virHashAddEntry(ctxt->group, key, value) < 0) {
         VIR_FREE(value);
