@@ -29,6 +29,7 @@
 #include "virfile.h"
 #include "virhash.h"
 #include "virthread.h"
+#include "virstring.h"
 
 #include <fcntl.h>
 #include <unistd.h>
@@ -69,10 +70,8 @@ static char *virLockSpaceGetResourcePath(virLockSpacePtr lockspace,
             return NULL;
         }
     } else {
-        if (!(ret = strdup(resname))) {
-            virReportOOMError();
+        if (VIR_STRDUP(ret, resname) < 0)
             return NULL;
-        }
     }
 
     return ret;
@@ -131,8 +130,8 @@ virLockSpaceResourceNew(virLockSpacePtr lockspace,
     res->fd = -1;
     res->flags = flags;
 
-    if (!(res->name = strdup(resname)))
-        goto no_memory;
+    if (VIR_STRDUP(res->name, resname) < 0)
+        goto error;
 
     if (!(res->path = virLockSpaceGetResourcePath(lockspace, resname)))
         goto no_memory;
@@ -261,9 +260,8 @@ virLockSpacePtr virLockSpaceNew(const char *directory)
         return NULL;
     }
 
-    if (directory &&
-        !(lockspace->dir = strdup(directory)))
-        goto no_memory;
+    if (VIR_STRDUP(lockspace->dir, directory) < 0)
+        goto error;
 
     if (!(lockspace->resources = virHashCreate(VIR_LOCKSPACE_TABLE_SIZE,
                                                virLockSpaceResourceDataFree)))
@@ -289,8 +287,6 @@ virLockSpacePtr virLockSpaceNew(const char *directory)
 
     return lockspace;
 
-no_memory:
-    virReportOOMError();
 error:
     virLockSpaceFree(lockspace);
     return NULL;
@@ -323,10 +319,8 @@ virLockSpacePtr virLockSpaceNewPostExecRestart(virJSONValuePtr object)
 
     if (virJSONValueObjectHasKey(object, "directory")) {
         const char *dir = virJSONValueObjectGetString(object, "directory");
-        if (!(lockspace->dir = strdup(dir))) {
-            virReportOOMError();
+        if (VIR_STRDUP(lockspace->dir, dir) < 0)
             goto error;
-        }
     }
 
     if (!(resources = virJSONValueObjectGet(object, "resources"))) {
@@ -341,7 +335,7 @@ virLockSpacePtr virLockSpaceNewPostExecRestart(virJSONValuePtr object)
         goto error;
     }
 
-    for (i = 0 ; i < n ; i++) {
+    for (i = 0; i < n; i++) {
         virJSONValuePtr child = virJSONValueArrayGet(resources, i);
         virLockSpaceResourcePtr res;
         const char *tmp;
@@ -361,8 +355,7 @@ virLockSpacePtr virLockSpaceNewPostExecRestart(virJSONValuePtr object)
             virLockSpaceResourceFree(res);
             goto error;
         }
-        if (!(res->name = strdup(tmp))) {
-            virReportOOMError();
+        if (VIR_STRDUP(res->name, tmp) < 0) {
             virLockSpaceResourceFree(res);
             goto error;
         }
@@ -373,8 +366,7 @@ virLockSpacePtr virLockSpaceNewPostExecRestart(virJSONValuePtr object)
             virLockSpaceResourceFree(res);
             goto error;
         }
-        if (!(res->path = strdup(tmp))) {
-            virReportOOMError();
+        if (VIR_STRDUP(res->path, tmp) < 0) {
             virLockSpaceResourceFree(res);
             goto error;
         }
@@ -425,7 +417,7 @@ virLockSpacePtr virLockSpaceNewPostExecRestart(virJSONValuePtr object)
             goto error;
         }
 
-        for (j = 0 ; j < res->nOwners ; j++) {
+        for (j = 0; j < res->nOwners; j++) {
             unsigned long long int owner;
             virJSONValuePtr ownerval = virJSONValueArrayGet(owners, j);
 
@@ -516,7 +508,7 @@ virJSONValuePtr virLockSpacePreExecRestart(virLockSpacePtr lockspace)
             goto error;
         }
 
-        for (i = 0 ; i < res->nOwners ; i++) {
+        for (i = 0; i < res->nOwners; i++) {
             virJSONValuePtr owner = virJSONValueNewNumberUlong(res->owners[i]);
             if (!owner)
                 goto error;
@@ -700,7 +692,7 @@ int virLockSpaceReleaseResource(virLockSpacePtr lockspace,
         goto cleanup;
     }
 
-    for (i = 0 ; i < res->nOwners ; i++) {
+    for (i = 0; i < res->nOwners; i++) {
         if (res->owners[i] == owner) {
             break;
         }
@@ -748,7 +740,7 @@ virLockSpaceRemoveResourcesForOwner(const void *payload,
 
     VIR_DEBUG("res %s owner %lld", res->name, (unsigned long long)data->owner);
 
-    for (i = 0 ; i < res->nOwners ; i++) {
+    for (i = 0; i < res->nOwners; i++) {
         if (res->owners[i] == data->owner) {
             break;
         }
