@@ -226,6 +226,26 @@ virSocketAddrIsPrivate(const virSocketAddrPtr addr)
     return false;
 }
 
+/*
+ * virSocketAddrIsWildcard:
+ * @addr: address to check
+ *
+ * Check if passed address is a variant of ANYCAST address.
+ */
+bool
+virSocketAddrIsWildcard(const virSocketAddrPtr addr)
+{
+    struct in_addr tmp = { .s_addr = INADDR_ANY };
+    switch (addr->data.stor.ss_family) {
+    case AF_INET:
+        return memcmp(&addr->data.inet4.sin_addr.s_addr, &tmp.s_addr,
+                      sizeof(addr->data.inet4.sin_addr.s_addr)) == 0;
+    case AF_INET6:
+        return IN6_IS_ADDR_UNSPECIFIED(&addr->data.inet6.sin6_addr);
+    }
+    return false;
+}
+
 /*
  * virSocketAddrFormat:
  * @addr: an initialized virSocketAddrPtr
