@@ -31,7 +31,6 @@
 
 #define VIR_FROM_THIS VIR_FROM_NONE
 
-VIR_ENUM_DECL(virTypedParameter)
 VIR_ENUM_IMPL(virTypedParameter, VIR_TYPED_PARAM_LAST,
               "unknown",
               "int",
@@ -48,7 +47,7 @@ VIR_ENUM_IMPL(virTypedParameter, VIR_TYPED_PARAM_LAST,
  * parameters.  Return 0 on success, -1 on failure with error message
  * already issued.  */
 int
-virTypedParameterArrayValidate(virTypedParameterPtr params, int nparams, ...)
+virTypedParamsValidate(virTypedParameterPtr params, int nparams, ...)
 {
     va_list ap;
     int ret = -1;
@@ -87,7 +86,7 @@ virTypedParameterArrayValidate(virTypedParameterPtr params, int nparams, ...)
             name = va_arg(ap, const char *);
         }
         if (!name) {
-            virReportError(VIR_ERR_INVALID_ARG,
+            virReportError(VIR_ERR_ARGUMENT_UNSUPPORTED,
                            _("parameter '%s' not supported"),
                            params[i].field);
             goto cleanup;
@@ -109,6 +108,78 @@ cleanup:
 
 }
 
+/* Check if params contains only specified parameter names. Return true if
+ * only specified names are present in params, false if params contains any
+ * unspecified parameter name. */
+bool
+virTypedParamsCheck(virTypedParameterPtr params,
+                    int nparams,
+                    const char **names,
+                    int nnames)
+{
+    int i, j;
+
+    for (i = 0; i < nparams; i++) {
+        bool found = false;
+        for (j = 0; j < nnames; j++) {
+            if (STREQ(params[i].field, names[j])) {
+                found = true;
+                break;
+            }
+        }
+        if (!found)
+            return false;
+    }
+
+    return true;
+}
+
+char *
+virTypedParameterToString(virTypedParameterPtr param)
+{
+    char *value;
+    int ret = -1;
+
+    switch (param->type) {
+    case VIR_TYPED_PARAM_INT:
+        if ((ret = virAsprintf(&value, "%d", param->value.i)) < 0)
+            virReportOOMError();
+        break;
+    case VIR_TYPED_PARAM_UINT:
+        if ((ret = virAsprintf(&value, "%u", param->value.ui)) < 0)
+            virReportOOMError();
+        break;
+    case VIR_TYPED_PARAM_LLONG:
+        if ((ret = virAsprintf(&value, "%lld", param->value.l)) < 0)
+            virReportOOMError();
+        break;
+    case VIR_TYPED_PARAM_ULLONG:
+        if ((ret = virAsprintf(&value, "%llu", param->value.ul)) < 0)
+            virReportOOMError();
+        break;
+    case VIR_TYPED_PARAM_DOUBLE:
+        if ((ret = virAsprintf(&value, "%g", param->value.d)) < 0)
+            virReportOOMError();
+        break;
+    case VIR_TYPED_PARAM_BOOLEAN:
+        if ((ret = virAsprintf(&value, "%d", param->value.b)) < 0)
+            virReportOOMError();
+        break;
+    case VIR_TYPED_PARAM_STRING:
+        ret = VIR_STRDUP(value, param->value.s);
+        break;
+    default:
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       _("unexpected type %d for field %s"),
+                       param->type, param->field);
+    }
+
+    if (ret < 0)
+        return NULL;
+    else
+        return value;
+}
+
 /* Assign name, type, and the appropriately typed arg to param; in the
  * case of a string, the caller is assumed to have malloc'd a string,
  * or can pass NULL to have this function malloc an empty string.
@@ -259,6 +330,105 @@ cleanup:
 }
 
 
+/**
+ * virTypedParamsReplaceString:
+ * @params: pointer to the array of typed parameters
+ * @nparams: number of parameters in the @params array
+ * @name: name of the parameter to set
+ * @value: the value to store into the parameter
+ *
+ * Sets new value @value to parameter called @name with char * type. If the
+ * parameter does not exist yet in @params, it is automatically created and
+ * @naprams is incremented by one. Otherwise current value of the parameter
+ * is freed on success. The function creates its own copy of @value string,
+ * which needs to be freed using virTypedParamsFree or virTypedParamsClear.
+ *
+ * Returns 0 on success, -1 on error.
+ */
+int
+virTypedParamsReplaceString(virTypedParameterPtr *params,
+                            int *nparams,
+                            const char *name,
+                            const char *value)
+{
+    char *str = NULL;
+    char *old = NULL;
+    size_t n = *nparams;
+    virTypedParameterPtr param;
+
+    virResetLastError();
+
+    param = virTypedParamsGet(*params, n, name);
+    if (param) {
+        if (param->type != VIR_TYPED_PARAM_STRING) {
+            virReportError(VIR_ERR_INVALID_ARG,
+                           _("Parameter '%s' is not a string"),
+                           param->field);
+            goto error;
+        }
+        old = param->value.s;
+    } else {
+        if (VIR_EXPAND_N(*params, n, 1) < 0) {
+            virReportOOMError();
+            goto error;
+        }
+        param = *params + n - 1;
+    }
+
+    if (VIR_STRDUP(str, value) < 0)
+        goto error;
+
+    if (virTypedParameterAssign(param, name,
+                                VIR_TYPED_PARAM_STRING, str) < 0) {
+        param->value.s = old;
+        VIR_FREE(str);
+        goto error;
+    }
+    VIR_FREE(old);
+
+    *nparams = n;
+    return 0;
+
+error:
+    virDispatchError(NULL);
+    return -1;
+}
+
+
+int
+virTypedParamsCopy(virTypedParameterPtr *dst,
+                   virTypedParameterPtr src,
+                   int nparams)
+{
+    int i;
+
+    *dst = NULL;
+    if (!src || nparams <= 0)
+        return 0;
+
+    if (VIR_ALLOC_N(*dst, nparams) < 0) {
+        virReportOOMError();
+        return -1;
+    }
+
+    for (i = 0; i < nparams; i++) {
+        ignore_value(virStrcpyStatic((*dst)[i].field, src[i].field));
+        (*dst)[i].type = src[i].type;
+        if (src[i].type == VIR_TYPED_PARAM_STRING) {
+            if (VIR_STRDUP((*dst)[i].value.s, src[i].value.s) < 0) {
+                virTypedParamsFree(*dst, i - 1);
+                *dst = NULL;
+                return -1;
+            }
+        } else {
+            (*dst)[i].value = src[i].value;
+        }
+    }
+
+    return 0;
+}
+
+
 /* The following APIs are public and their signature may never change. */
 
 /**
