@@ -36,6 +36,7 @@
 #include "virutil.h"
 #include "viralloc.h"
 #include "virfile.h"
+#include "virstring.h"
 
 #define VIR_FROM_THIS VIR_FROM_XML
 
@@ -86,11 +87,8 @@ virXPathString(const char *xpath,
         xmlXPathFreeObject(obj);
         return NULL;
     }
-    ret = strdup((char *) obj->stringval);
+    ignore_value(VIR_STRDUP(ret, (char *) obj->stringval));
     xmlXPathFreeObject(obj);
-    if (ret == NULL) {
-        virReportOOMError();
-    }
     return ret;
 }
 
