@@ -110,6 +110,8 @@ mymain(void)
     DO_TEST("pool-dir", "vol-file");
     DO_TEST("pool-dir", "vol-file-backing");
     DO_TEST("pool-dir", "vol-qcow2");
+    DO_TEST("pool-dir", "vol-qcow2-1.1");
+    DO_TEST("pool-dir", "vol-qcow2-lazy");
     DO_TEST("pool-disk", "vol-partition");
     DO_TEST("pool-logical", "vol-logical");
     DO_TEST("pool-logical", "vol-logical-backing");
