@@ -125,6 +125,8 @@ static int test2(const void *data ATTRIBUTE_UNUSED)
         goto error;
 
     virBitmapClearAll(bitmap);
+    if (!virBitmapIsAllClear(bitmap))
+        goto error;
     if (testBit(bitmap, 0, size - 1, false) < 0)
         goto error;
     if (virBitmapCountBits(bitmap) != 0)
@@ -154,6 +156,9 @@ static int test3(const void *data ATTRIBUTE_UNUSED)
     if (!virBitmapIsAllSet(bitmap))
         goto error;
 
+    virBitmapClearAll(bitmap);
+    if (!virBitmapIsAllClear(bitmap))
+        goto error;
     ret = 0;
 
 error:
@@ -196,6 +201,9 @@ static int test4(const void *data ATTRIBUTE_UNUSED)
     if (virBitmapNextClearBit(bitmap, i) != -1)
         goto error;
 
+    if (!virBitmapIsAllClear(bitmap))
+        goto error;
+
     virBitmapFree(bitmap);
     bitmap = NULL;
 
@@ -406,6 +414,10 @@ static int test7(const void *v ATTRIBUTE_UNUSED)
         if (!virBitmapIsAllSet(bitmap))
             goto error;
 
+        virBitmapClearAll(bitmap);
+        if (!virBitmapIsAllClear(bitmap))
+            goto error;
+
         virBitmapFree(bitmap);
     }
 
@@ -416,6 +428,31 @@ error:
     return -1;
 }
 
+static int test8(const void *v ATTRIBUTE_UNUSED)
+{
+    virBitmapPtr bitmap = NULL;
+    char data[108] = {0x00,};
+    int ret = -1;
+
+    bitmap = virBitmapNewData(data, sizeof(data));
+    if (!bitmap)
+        goto cleanup;
+
+    if (!virBitmapIsAllClear(bitmap))
+        goto cleanup;
+
+    if (virBitmapSetBit(bitmap, 11) < 0)
+        goto cleanup;
+
+    if (virBitmapIsAllClear(bitmap))
+        goto cleanup;
+
+    ret = 0;
+cleanup:
+    virBitmapFree(bitmap);
+    return ret;
+}
+
 static int
 mymain(void)
 {
@@ -435,7 +472,8 @@ mymain(void)
         ret = -1;
     if (virtTestRun("test7", 1, test7, NULL) < 0)
         ret = -1;
-
+    if (virtTestRun("test8", 1, test8, NULL) < 0)
+        ret = -1;
 
     return ret;
 }
