@@ -25,8 +25,10 @@
 #include "testutils.h"
 #include "vircommand.h"
 #include "virerror.h"
+#include "virfile.h"
 #include "virlog.h"
 #include "virstoragefile.h"
+#include "virstring.h"
 
 #define VIR_FROM_THIS VIR_FROM_NONE
 
